/*============================================================================*/
/*                                                                            */
/*                                                                            */
/*                            reLOC 0.10-jerusalem                            */
/*                                                                            */
/*                   (C) Copyright 2011-2013 Pavel Surynek                    */
/*            http://www.surynek.com | <pavel.surynek@mff.cuni.cz>            */
/*                                                                            */
/*                                                                            */
/*============================================================================*/
// reloc.cpp / 0.10-jerusalem_140
/*----------------------------------------------------------------------------*/
//
// Relocation problem solving package - original development source.
//
/*----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

#include "config.h"
#include "compile.h"
#include "defs.h"
#include "result.h"
#include "reloc.h"

using namespace sReloc;


/*----------------------------------------------------------------------------*/

namespace sReloc
{


/*----------------------------------------------------------------------------*/
// Global constants

    const sString sRELOC_INDENT = "    ";


/*----------------------------------------------------------------------------*/
// sVertex

    sVertex::sVertex()
	: m_id(0)
    {
	// nothing
    }


    sVertex::sVertex(int id)
	: m_id(id)
    {
	// nothing
    }


    sVertex::sVertex(const sVertex &vertex)
	: m_id(vertex.m_id)
    {
	sASSERT(m_Neighbors.empty() && vertex.m_Neighbors.empty());
    }


    const sVertex& sVertex::operator=(const sVertex &vertex)
    {
	m_id = vertex.m_id;
	sASSERT(m_Neighbors.empty() && vertex.m_Neighbors.empty());

	return *this;
    }


    int sVertex::calc_NeighborCount(void) const
    {
	return m_Neighbors.size();
    }


    int sVertex::calc_NeighborOrder(int vertex_id) const
    {
	int order = 0;
	for (Neighbors_list::const_iterator neighbor = m_Neighbors.begin(); neighbor != m_Neighbors.end(); ++neighbor)
	{
	    if ((*neighbor)->m_target->m_id == vertex_id)
	    {
		return order; 
	    }
	    ++order;
	}

	return ORDER_UNDEFINED;
    }


    int sVertex::calc_NeighborID(int order) const
    {
	Neighbors_list::const_iterator neighbor = m_Neighbors.begin();

	while (--order >= 0)
	{
	    ++neighbor;
	}

	return (*neighbor)->m_target->m_id;
    }


    void sVertex::to_Screen(const sString &indent) const
    {
	to_Stream(stdout, indent);
    }


    void sVertex::to_Stream(FILE *fw, const sString &indent) const
    {
	fprintf(fw, "%sVertex: (id = %d)", indent.c_str(), m_id);
	
	if (!m_Neighbors.empty())
	{
	    fprintf(fw, " {");
	    for (Neighbors_list::const_iterator neighbor = m_Neighbors.begin(); neighbor != m_Neighbors.end(); ++neighbor)
	    {
		fprintf(fw, "%d ", (*neighbor)->m_target->m_id, (*neighbor)->m_source->m_id);
	    }
	    fprintf(fw, "}");

	    fprintf(fw, " i[");
	    for (Neighbors_list::const_iterator neighbor = m_in_Neighbors.begin(); neighbor != m_in_Neighbors.end(); ++neighbor)
	    {
		fprintf(fw, "%d ", (*neighbor)->m_source->m_id);
	    }
	    fprintf(fw, "]");

	    fprintf(fw, " o[");
	    for (Neighbors_list::const_iterator neighbor = m_out_Neighbors.begin(); neighbor != m_out_Neighbors.end(); ++neighbor)
	    {
		fprintf(fw, "%d ", (*neighbor)->m_target->m_id);
	    }
	    fprintf(fw, "]\n");
	}
	else
	{
	    fprintf(fw, "\n");
	}
    }


/*----------------------------------------------------------------------------*/
// sArc

    sArc::sArc()
	: m_edge(NULL)
	, m_source(NULL)
	, m_target(NULL)
    {
	// nothing
    }


    sArc::sArc(sEdge *edge, sVertex *source, sVertex *target)
	: m_edge(edge)
	, m_source(source)
	, m_target(target)
    {
	// nothing
    }


    sArc::sArc(const sArc &arc)
	: m_edge(arc.m_edge)
	, m_source(arc.m_source)
	, m_target(arc.m_target)
    {
	// nothing
    }


    const sArc& sArc::operator=(const sArc &arc)
    {
	m_edge = arc.m_edge;
	m_source = arc.m_source;
	m_target = arc.m_target;

	return *this;
    }


    void sArc::to_Screen(const sString &indent) const
    {
	to_Stream(stdout, indent);
    }

	 
    void sArc::to_Stream(FILE *fw, const sString &indent) const
    {
	fprintf(fw, "%sArc: (edge = %p, source = %p, target = %p)\n", indent.c_str(), (void*)m_edge, (void*)m_source, (void*)m_target);
    }


/*----------------------------------------------------------------------------*/
// sEdge

    sEdge::sEdge(int id, sVertex *vertex_u, sVertex *vertex_v, bool directed)
	: m_id(id)
	, m_directed(directed)
	, m_arc_uv(this, vertex_u, vertex_v)
	, m_arc_vu(this, vertex_v, vertex_u)
    {
	// nothing
    }


    sEdge::sEdge(const sEdge &edge)
	: m_id(edge.m_id)
	, m_directed(edge.m_directed)
	, m_arc_uv(this, edge.m_arc_uv.m_source, edge.m_arc_uv.m_target)
	, m_arc_vu(this, edge.m_arc_vu.m_source, edge.m_arc_vu.m_target)
    {
	// nothing
    }


    const sEdge& sEdge::operator=(const sEdge &edge)
    {
	m_id = edge.m_id;
	m_directed = edge.m_directed;

	m_arc_uv = sArc(this, edge.m_arc_uv.m_source, edge.m_arc_uv.m_target);
	m_arc_vu = sArc(this, edge.m_arc_vu.m_source, edge.m_arc_vu.m_target);

	return *this;
    }


    void sEdge::to_Screen(const sString &indent) const
    {
	to_Stream(stdout, indent);
    }


    void sEdge::to_Stream(FILE *fw, const sString &indent) const
    {
	if (m_directed)
	{
	    fprintf(fw, "%s Edge  %d: %d --> %d\n", indent.c_str(), m_id, m_arc_uv.m_source->m_id, m_arc_uv.m_target->m_id);
	    fprintf(fw, "%s[Edge  %d: %d --> %d]\n", indent.c_str(), m_id, m_arc_vu.m_source->m_id, m_arc_vu.m_target->m_id);
	}
	else
	{
	    fprintf(fw, "%s Edge  %d: %d <-> %d\n", indent.c_str(), m_id, m_arc_uv.m_source->m_id, m_arc_uv.m_target->m_id);
	    fprintf(fw, "%s[Edge  %d: %d <-> %d]\n", indent.c_str(), m_id, m_arc_vu.m_source->m_id, m_arc_vu.m_target->m_id);
	}
/*
	fprintf(fw, "%s]\n", indent.c_str());
	m_arc_uv.to_Stream(fw, indent + sRELOC_INDENT);
	m_arc_vu.to_Stream(fw, indent + sRELOC_INDENT);
	fprintf(fw, "%s]\n", indent.c_str());
*/
    }


/*----------------------------------------------------------------------------*/
// sUndirectedGraph

    sUndirectedGraph::sUndirectedGraph()
	: m_Edge_cnt(0)
	, m_all_pairs_distances_calculated(false)
	, m_all_pairs_coop_distances_calculated(false)
    {
	// nothing
    }


    sUndirectedGraph::sUndirectedGraph(int x_size, int y_size)
	: m_Edge_cnt(0)
	, m_all_pairs_distances_calculated(false)
	, m_all_pairs_coop_distances_calculated(false)
    {
	add_Vertices(x_size * y_size);

	for (int i = 0; i < x_size - 1; ++i)
	{
	    for (int j = 0; j < y_size - 1; ++j)
	    {
		int u_id = j * x_size + i;
		int v_id = (j + 1) * x_size + i;
		int w_id = j * x_size + i + 1;

		add_Edge(u_id, v_id);
		add_Edge(u_id, w_id);		
	    }
	}
	for (int i = 0; i < x_size - 1; ++i)
	{
	    int u_id = (y_size - 1) * x_size + i;
	    int v_id = (y_size - 1) * x_size + (i + 1);

	    add_Edge(u_id, v_id);
	}
	for (int j = 0; j < y_size - 1; ++j)
	{
	    int u_id = (j + 1) * x_size - 1;
	    int v_id = (j + 2) * x_size - 1;

	    add_Edge(u_id, v_id);
	}
    }


    sUndirectedGraph::sUndirectedGraph(int x_size, int y_size, double obstacle_prob)
	: m_Edge_cnt(0)
	, m_all_pairs_distances_calculated(false)
	, m_all_pairs_coop_distances_calculated(false)
    {
	int *matrix = new int[x_size * y_size];
	int cnt = 0;

	int rnd_limit = RAND_MAX * obstacle_prob;

	for (int i = 0; i < x_size; ++i)
	{
	    for (int j = 0; j < y_size; ++j)
	    {
		int rnd = rand();

		if (rnd > rnd_limit)
		{
		    matrix[j * x_size + i] = cnt++;
		}
		else
		{
		    matrix[j * x_size + i] = -1;
		}
	    }
	}

	add_Vertices(cnt);

	for (int i = 0; i < x_size - 1; ++i)
	{
	    for (int j = 0; j < y_size - 1; ++j)
	    {
		int u_id = j * x_size + i;
		int v_id = (j + 1) * x_size + i;
		int w_id = j * x_size + i + 1;

		if (matrix[u_id] != -1)
		{
		    if (matrix[v_id] != -1)
		    {
			add_Edge(matrix[u_id], matrix[v_id]);
		    }
		    if (matrix[w_id] != -1)
		    {
			add_Edge(matrix[u_id], matrix[w_id]);
		    }
		}
	    }
	}
	for (int i = 0; i < x_size - 1; ++i)
	{
	    int u_id = (y_size - 1) * x_size + i;
	    int v_id = (y_size - 1) * x_size + (i + 1);

	    if (matrix[u_id] != -1 && matrix[v_id] != -1)
	    {
		add_Edge(matrix[u_id], matrix[v_id]);
	    }
	}
	for (int j = 0; j < y_size - 1; ++j)
	{
	    int u_id = (j + 1) * x_size - 1;
	    int v_id = (j + 2) * x_size - 1;

	    if (matrix[u_id] != -1 && matrix[v_id] != -1)
	    {
		add_Edge(matrix[u_id], matrix[v_id]);
	    }
	}
	delete matrix;
    }


    sUndirectedGraph::sUndirectedGraph(const sUndirectedGraph &undirected_graph)
	: m_Edge_cnt(0)
    {
	add_Vertices(undirected_graph.m_Vertices.size());

	for (Edges_list::const_iterator edge = undirected_graph.m_Edges.begin(); edge != undirected_graph.m_Edges.end(); ++edge)	
	{
	    if (edge->m_directed)
	    {
		add_Arrow(edge->m_arc_uv.m_source->m_id, edge->m_arc_uv.m_target->m_id);
	    }
	    else
	    {
		add_Edge(edge->m_arc_uv.m_source->m_id, edge->m_arc_uv.m_target->m_id);
	    }
	}
    }


    const sUndirectedGraph& sUndirectedGraph::operator=(const sUndirectedGraph &undirected_graph)
    {
	m_Edge_cnt = 0;

	m_Vertices.clear();
	m_Edges.clear();

	add_Vertices(undirected_graph.m_Vertices.size());

	for (Edges_list::const_iterator edge = undirected_graph.m_Edges.begin(); edge != undirected_graph.m_Edges.end(); ++edge)
	{
	    if (edge->m_directed)
	    {
		add_Arrow(edge->m_arc_uv.m_source->m_id, edge->m_arc_uv.m_target->m_id);
	    }
	    else
	    {
		add_Edge(edge->m_arc_uv.m_source->m_id, edge->m_arc_uv.m_target->m_id);
	    }
	}	
	return *this;
    }
    

    void sUndirectedGraph::add_Vertex(void)
    {
	m_Vertices.push_back(sVertex(m_Vertices.size()));
    }


    void sUndirectedGraph::add_Vertices(int Vertex_cnt)
    {
	while (Vertex_cnt-- > 0)
	{
	    m_Vertices.push_back(sVertex(m_Vertices.size()));
	}
    }


    int sUndirectedGraph::get_VertexCount(void) const
    {
	return m_Vertices.size();
    }


    sVertex* sUndirectedGraph::get_Vertex(int id)
    {
	return &m_Vertices[id];
    }


    const sVertex* sUndirectedGraph::get_Vertex(int id) const
    {
	return &m_Vertices[id];
    }


    int sUndirectedGraph::get_EdgeCount(void) const
    {
	return m_Edge_cnt;
	//	return m_Edges.size();
    }


    void sUndirectedGraph::add_Arrow(int u_id, int v_id)
    {
	sASSERT(m_Vertices.size() > u_id && m_Vertices.size() > v_id);
	sASSERT(u_id != v_id);

	int id = m_Edge_cnt++;
	m_Edges.push_back(sEdge(id, &m_Vertices[u_id], &m_Vertices[v_id], true));
	sEdge &inserted_edge = m_Edges.back();

	m_Vertices[u_id].m_Neighbors.push_back(&inserted_edge.m_arc_uv);
	m_Vertices[v_id].m_in_Neighbors.push_back(&inserted_edge.m_arc_uv);
	m_Vertices[u_id].m_out_Neighbors.push_back(&inserted_edge.m_arc_uv);
    }


    void sUndirectedGraph::add_Edge(int u_id, int v_id)
    {
	sASSERT(m_Vertices.size() > u_id && m_Vertices.size() > v_id);
	sASSERT(u_id != v_id);

	int id = m_Edge_cnt++;
	m_Edges.push_back(sEdge(id, &m_Vertices[u_id], &m_Vertices[v_id]));
	sEdge &inserted_edge = m_Edges.back();	

	m_Vertices[u_id].m_Neighbors.push_back(&inserted_edge.m_arc_uv);
	m_Vertices[v_id].m_in_Neighbors.push_back(&inserted_edge.m_arc_uv);
	m_Vertices[u_id].m_out_Neighbors.push_back(&inserted_edge.m_arc_uv);
	
	m_Vertices[v_id].m_Neighbors.push_back(&inserted_edge.m_arc_vu);
	m_Vertices[u_id].m_in_Neighbors.push_back(&inserted_edge.m_arc_vu);
	m_Vertices[v_id].m_out_Neighbors.push_back(&inserted_edge.m_arc_vu);
    }


    bool sUndirectedGraph::is_Adjacent(int u_id, int v_id) const
    {
	sASSERT(m_Vertices.size() > u_id && m_Vertices.size() > v_id);

	if (m_Vertices[u_id].m_Neighbors.size() < m_Vertices[v_id].m_Neighbors.size())
	{
	    for (sVertex::Neighbors_list::const_iterator u_neighbor = m_Vertices[u_id].m_Neighbors.begin(); u_neighbor != m_Vertices[u_id].m_Neighbors.end(); ++u_neighbor)
	    {
		if ((*u_neighbor)->m_target->m_id == v_id)
		{
		    return true;
		}
	    }
	}
	else
	{
	    for (sVertex::Neighbors_list::const_iterator v_neighbor = m_Vertices[v_id].m_Neighbors.begin(); v_neighbor != m_Vertices[v_id].m_Neighbors.end(); ++v_neighbor)
	    {
		if ((*v_neighbor)->m_target->m_id == u_id)
		{
		    return true;
		}
	    }
	}

	return false;
    }


    bool sUndirectedGraph::is_LinkedTo(int u_id, int v_id) const
    {
	for (sVertex::Neighbors_list::const_iterator u_neighbor = m_Vertices[u_id].m_out_Neighbors.begin(); u_neighbor != m_Vertices[u_id].m_out_Neighbors.end(); ++u_neighbor)
	{
	    if ((*u_neighbor)->m_target->m_id == v_id)
	    {
		return true;
	    }
	}
	return false;
    }


    int sUndirectedGraph::calc_ShortestPath(int u_id, int v_id) const
    {
	Distances_vector Distances;
	calc_SingleSourceShortestPaths(u_id, Distances);

	return Distances[v_id];
    }


    void sUndirectedGraph::calc_SingleSourceShortestPaths(int s_id, Distances_vector &Distances) const
    {
	Distances.resize(m_Vertices.size(), sINT_32_MAX);

	Distances[s_id] = 0;
	VertexQueue_multimap vertex_Queue;
	vertex_Queue.insert(VertexQueue_multimap::value_type(0, s_id));

	while (!vertex_Queue.empty())
	{
	    VertexQueue_multimap::value_type front_record = *vertex_Queue.begin();

	    for (sVertex::Neighbors_list::const_iterator neighbor = m_Vertices[front_record.second].m_Neighbors.begin(); neighbor != m_Vertices[front_record.second].m_Neighbors.end(); ++neighbor)
	    {
		int neighbor_id = (*neighbor)->m_target->m_id;
		int distance_update = front_record.first + 1;
		if (Distances[neighbor_id] == sINT_32_MAX || Distances[neighbor_id] > distance_update)
		{
		    Distances[neighbor_id] = distance_update;
		    vertex_Queue.insert(VertexQueue_multimap::value_type(distance_update, neighbor_id));
		}
	    }
	    vertex_Queue.erase(vertex_Queue.begin());
	}
    }


    int sUndirectedGraph::calc_ShortestCoopPath(int u1_id, int u2_id, int v1_id, int v2_id) const
    {
	Distances_2d_vector Distances;
	calc_SingleSourceShortestCoopPaths(u1_id, u2_id, Distances);

	return Distances[v1_id][v2_id];
    }


    void sUndirectedGraph::calc_SingleSourceShortestCoopPaths(int s1_id, int s2_id, Distances_2d_vector &Distances) const
    {
	sASSERT(s1_id != s2_id);

	int N_Vertices = m_Vertices.size();
	Distances.resize(N_Vertices);
	for (int i = 0; i < N_Vertices; ++i)
	{
	    Distances[i].resize(N_Vertices, sINT_32_MAX);
	}

	Distances[s1_id][s2_id] = 0;
	VertexPairQueue_multimap vertex_Queue;
	vertex_Queue.insert(VertexPairQueue_multimap::value_type(0, Vertex_pair(s1_id, s2_id)));

	while (!vertex_Queue.empty())
	{
	    VertexPairQueue_multimap::value_type front_record = *vertex_Queue.begin();
	    
	    VertexIDs_vector Neighbors_1;
	    Neighbors_1.push_back(front_record.second.first);
	    for (sVertex::Neighbors_list::const_iterator neighbor_1 = m_Vertices[front_record.second.first].m_Neighbors.begin(); neighbor_1 != m_Vertices[front_record.second.first].m_Neighbors.end(); ++neighbor_1)
	    {
		Neighbors_1.push_back((*neighbor_1)->m_target->m_id);
	    }

	    VertexIDs_vector Neighbors_2;
	    Neighbors_2.push_back(front_record.second.second);
	    for (sVertex::Neighbors_list::const_iterator neighbor_2 = m_Vertices[front_record.second.second].m_Neighbors.begin(); neighbor_2 != m_Vertices[front_record.second.second].m_Neighbors.end(); ++neighbor_2)
	    {
		Neighbors_2.push_back((*neighbor_2)->m_target->m_id);
	    }

	    for (VertexIDs_vector::const_iterator neighbor_1 = Neighbors_1.begin(); neighbor_1 != Neighbors_1.end(); ++neighbor_1)
	    {
		for (VertexIDs_vector::const_iterator neighbor_2 = Neighbors_2.begin(); neighbor_2 != Neighbors_2.end(); ++neighbor_2)
		{
		    int distance_update = front_record.first + 1;
		    bool update = false;

		    if (front_record.second.first != *neighbor_1)
		    {
			if (front_record.second.second != *neighbor_2)
			{
			    std::set<int, std::less<int> > context;
			    context.insert(front_record.second.first);
			    context.insert(front_record.second.second);
			    context.insert(*neighbor_1);
			    context.insert(*neighbor_2);

			    if (context.size() == 4)
			    {
				update = true;
			    }
			}
			else
			{
			    std::set<int, std::less<int> > context;
			    context.insert(front_record.second.first);
			    context.insert(front_record.second.second);
			    context.insert(*neighbor_1);

			    if (context.size() == 3)
			    {
				update = true;
			    }
			}
		    }
		    else
		    {
			if (front_record.second.second != *neighbor_2)
			{
			    std::set<int, std::less<int> > context;
			    context.insert(front_record.second.first);
			    context.insert(front_record.second.second);
			    context.insert(*neighbor_2);

			    if (context.size() == 3)
			    {
				update = true;
			    }
			}
			else
			{
			    std::set<int, std::less<int> > context;
			    context.insert(front_record.second.first);
			    context.insert(front_record.second.second);

			    if (context.size() == 2)
			    {
				update = true;
			    }
			}
		    }
		    if (update)
		    {
			if (Distances[*neighbor_1][*neighbor_2] == sINT_32_MAX || Distances[*neighbor_1][*neighbor_2] > distance_update)
			{
			    Distances[*neighbor_1][*neighbor_2] = distance_update;
			    vertex_Queue.insert(VertexPairQueue_multimap::value_type(distance_update, Vertex_pair(*neighbor_1, *neighbor_2)));
			}
		    }
		}
	    }
	    vertex_Queue.erase(vertex_Queue.begin());
	}
    }


    void sUndirectedGraph::calc_AllPairsShortestPaths(Distances_2d_vector &all_pairs_Distances) const
    {
	all_pairs_Distances.clear();
	int N_Vertices = m_Vertices.size();

	for (int source_id = 0; source_id < N_Vertices; ++source_id)
	{
	    Distances_vector source_Distances;
	    calc_SingleSourceShortestPaths(source_id, source_Distances);

	    all_pairs_Distances.push_back(source_Distances);
	}
    }


    void sUndirectedGraph::calc_AllPairsShortestPaths(void)
    {
	if (!m_all_pairs_distances_calculated)
	{
	    calc_AllPairsShortestPaths(m_all_pairs_Distances);
	    m_all_pairs_distances_calculated = true;
	}
    }


    const sUndirectedGraph::Distances_2d_vector& sUndirectedGraph::get_AllPairsShortestPaths(void) const
    {
	sASSERT(m_all_pairs_distances_calculated);
	return m_all_pairs_Distances;
    }


    void sUndirectedGraph::calc_AllPairsShortestCoopPaths(Distances_4d_vector &all_pairs_Distances) const
    {
	all_pairs_Distances.clear();
	int N_Vertices = m_Vertices.size();

	for (int source_1_id = 0; source_1_id < N_Vertices; ++source_1_id)
	{
	    all_pairs_Distances.push_back(Distances_3d_vector());

	    for (int source_2_id = 0; source_2_id < N_Vertices; ++source_2_id)
	    {
		if (source_1_id != source_2_id)
		{
		    Distances_2d_vector source_Distances;
		    calc_SingleSourceShortestCoopPaths(source_1_id, source_2_id, source_Distances);
		    
		    all_pairs_Distances[source_1_id].push_back(source_Distances);
		}
		else
		{
		    Distances_2d_vector source_Distances;

		    source_Distances.resize(N_Vertices);
		    for (int i = 0; i < N_Vertices; ++i)
		    {
			source_Distances[i].resize(N_Vertices, sINT_32_MAX);
		    }
		    all_pairs_Distances[source_1_id].push_back(source_Distances);
		}
	    }
	}
    }


    void sUndirectedGraph::calc_AllPairsShortestCoopPaths(void)
    {
	if (!m_all_pairs_coop_distances_calculated)
	{
	    calc_AllPairsShortestCoopPaths(m_all_pairs_coop_Distances);
	    m_all_pairs_coop_distances_calculated = true;
	}
    }


    const sUndirectedGraph::Distances_4d_vector& sUndirectedGraph::get_AllPairsShortestCoopPaths(void) const
    {
	sASSERT(m_all_pairs_coop_distances_calculated);
	return m_all_pairs_coop_Distances;
    }


    void sUndirectedGraph::build_SpanningTree(int root_id, sUndirectedGraph &spanning_tree)
    {
	for (Vertices_vector::iterator vertex = m_Vertices.begin(); vertex != m_Vertices.end(); ++vertex)
	{
	    vertex->m_visited = false;
	}
	spanning_tree = sUndirectedGraph();
	spanning_tree.add_Vertices(get_VertexCount());

	build_SpanningTreeDFS(root_id, spanning_tree);
	/*
	for (Vertices_vector::iterator vertex = m_Vertices.begin(); vertex != m_Vertices.end(); ++vertex)
	{
	    for (sVertex::Neighbors_list::iterator neighbor = vertex->m_Neighbors.begin(); neighbor != vertex->m_Neighbors.end(); ++neighbor)
	    {
		if (!spanning_tree.is_Adjacent(vertex->m_id, (*neighbor)->m_target->m_id))
		{
		    double p = (double)rand() / RAND_MAX;		
		    if (p < 0.3)
		    {
			spanning_tree.add_Edge(vertex->m_id, (*neighbor)->m_target->m_id);
		    }
		}
		
	    }	
	}
	*/
	/*
	for (Vertices_vector::iterator vertex = m_Vertices.begin(); vertex != m_Vertices.end(); ++vertex)
	{
	    vertex->m_visited = false;
	}
	build_SpanningTreeDFS(root_id, spanning_tree);
	for (Vertices_vector::iterator vertex = m_Vertices.begin(); vertex != m_Vertices.end(); ++vertex)
	{
	    vertex->m_visited = false;
	}
	build_SpanningTreeDFS(root_id, spanning_tree);
	*/
    }


    void sUndirectedGraph::build_SpanningTreeDFS(int root_id, sUndirectedGraph &spanning_tree)
    {
	sVertex &vertex = m_Vertices[root_id];
	vertex.m_visited = true;

	int N_unvisited = 0;
	for (sVertex::Neighbors_list::iterator neighbor = vertex.m_Neighbors.begin(); neighbor != vertex.m_Neighbors.end(); ++neighbor)
	{
	    if (!(*neighbor)->m_target->m_visited)
	    {
		++N_unvisited;
	    }
	}
	while (N_unvisited > 0)
	{
	    int selected = rand() % N_unvisited;

	    for (sVertex::Neighbors_list::iterator neighbor = vertex.m_Neighbors.begin(); neighbor != vertex.m_Neighbors.end(); ++neighbor)
	    {
		if (!(*neighbor)->m_target->m_visited)
		{
		    if (selected == 0)
		    {		    
			if (!spanning_tree.is_Adjacent(root_id, (*neighbor)->m_target->m_id))
			{
			    spanning_tree.add_Edge(root_id, (*neighbor)->m_target->m_id);
//			    spanning_tree.add_Arrow(root_id, (*neighbor)->m_target->m_id);
			}
			build_SpanningTreeDFS((*neighbor)->m_target->m_id, spanning_tree);
			break;
		    }
		    else
		    {
			--selected;
		    }
		}
	    }
	    N_unvisited = 0;

	    for (sVertex::Neighbors_list::iterator neighbor = vertex.m_Neighbors.begin(); neighbor != vertex.m_Neighbors.end(); ++neighbor)
	    {
		if (!(*neighbor)->m_target->m_visited)
		{
		    ++N_unvisited;
		}
	    }
	}
    }


/*----------------------------------------------------------------------------*/

    void sUndirectedGraph::find_ShortestPath(int u_id, int v_id, VertexIDs_list &path)
    {
	for (Vertices_vector::iterator vertex = m_Vertices.begin(); vertex != m_Vertices.end(); ++vertex)
	{
	    vertex->m_distance = INT_MAX;
	    vertex->m_prev_id = -1;
	}
	find_ShortestPathBFS(u_id, v_id);

	int path_id = get_Vertex(v_id)->m_id;
	while (path_id != -1)
	{
	    path.push_front(path_id);
	    path_id = get_Vertex(path_id)->m_prev_id;
	}
    }


    void sUndirectedGraph::find_ShortestPathBFS(int u_id, int v_id)
    {
	Vertices_list queue;

	sVertex *first_vertex = get_Vertex(u_id);
	first_vertex->m_distance = 0;
	first_vertex->m_prev_id = -1;
	queue.push_back(first_vertex);

	while (!queue.empty())
	{
	    sVertex *vertex = queue.front();
	    queue.pop_front();

	    for (sVertex::Neighbors_list::iterator neighbor = vertex->m_Neighbors.begin(); neighbor != vertex->m_Neighbors.end(); ++neighbor)
	    {
		if ((*neighbor)->m_target->m_distance > vertex->m_distance + 1)
		{
		    (*neighbor)->m_target->m_distance = vertex->m_distance + 1;
		    (*neighbor)->m_target->m_prev_id = vertex->m_id;

		    queue.push_back((*neighbor)->m_target);

		    if ((*neighbor)->m_target->m_id == v_id)
		    {
			return;
		    }
		}
	    }	    
	}
    }


/*----------------------------------------------------------------------------*/

    void sUndirectedGraph::build_SparseGraph(const VertexPairs_vector &vertex_Pairs, sUndirectedGraph &sparse_graph)
    {
	sparse_graph = sUndirectedGraph();
	sparse_graph.add_Vertices(get_VertexCount());

	for (VertexPairs_vector::const_iterator vertex_pair = vertex_Pairs.begin(); vertex_pair != vertex_Pairs.end(); ++vertex_pair)
	{
	    VertexIDs_list path;
	    find_ShortestPath(vertex_pair->first, vertex_pair->second, path);

	    VertexIDs_list::const_iterator path_vertex_1 = path.begin();
	    VertexIDs_list::const_iterator path_vertex_2 = path_vertex_1;
	    ++path_vertex_2;

	    while (path_vertex_2 != path.end())
	    {
		if (!sparse_graph.is_LinkedTo(*path_vertex_1, *path_vertex_2) && !sparse_graph.is_LinkedTo(*path_vertex_2, *path_vertex_1))
		{
//		    sparse_graph.add_Edge((*path_vertex_1), (*path_vertex_2));
		    sparse_graph.add_Arrow((*path_vertex_1), (*path_vertex_2));
		}
		++path_vertex_1;
		++path_vertex_2;
	    }
	}
    }


/*----------------------------------------------------------------------------*/

    void sUndirectedGraph::to_Screen(const sString &indent) const
    {
	to_Stream(stdout, indent);
    }


    void sUndirectedGraph::to_Screen_vertices(const sString &indent) const
    {
	to_Stream_vertices(stdout, indent);
    }


    void sUndirectedGraph::to_Stream(FILE *fw, const sString &indent) const
    {
	fprintf(fw, "%sUndirected graph: (|V|=%ld |E|=%ld) [\n", indent.c_str(), m_Vertices.size(), m_Edges.size());

	for (Vertices_vector::const_iterator vertex = m_Vertices.begin(); vertex != m_Vertices.end(); ++vertex)
	{
	    vertex->to_Stream(fw, indent + sRELOC_INDENT);
	}

	for (Edges_list::const_iterator edge = m_Edges.begin(); edge != m_Edges.end(); ++edge)
	{
	    edge->to_Stream(fw, indent + sRELOC_INDENT);
	}
	fprintf(fw, "%s]\n", indent.c_str());
    }


    void sUndirectedGraph::to_Stream_vertices(FILE *fw, const sString &indent) const
    {
	fprintf(fw, "%sUndirected graph: (|V|=%ld, |E|=%ld) [\n", indent.c_str(), m_Vertices.size(), m_Edges.size());

	for (Vertices_vector::const_iterator vertex = m_Vertices.begin(); vertex != m_Vertices.end(); ++vertex)
	{
	    vertex->to_Stream(fw, indent + sRELOC_INDENT);
	}
	fprintf(fw, "%s]\n", indent.c_str());
    }


    sResult sUndirectedGraph::from_File_multirobot(const sString &filename)
    {
	sResult result;
	FILE *fr;

	if ((fr = fopen(filename.c_str(), "r")) == NULL)
	{
	    return sUNDIRECTED_GRAPH_OPEN_ERROR;
	}
	
	result = from_Stream_multirobot(fr);
	if (sFAILED(result))
	{
	    fclose(fr);
	    return result;
	}

	fclose(fr);

	return sRESULT_SUCCESS;
    }


    sResult sUndirectedGraph::from_Stream_multirobot(FILE *fr)
    {
	m_Vertices.clear();
	m_Edges.clear();

	int c = fgetc(fr);

	while (c != 'V')
	{
	    if (c != '\n')
	    {
		while(fgetc(fr) != '\n');
	    }
	    c = fgetc(fr);
	}

	fscanf(fr, " =\n");
	c = fgetc(fr);

	while (c == '(')
	{
	    add_Vertex();
	    if (c != '\n')
	    {
		while(fgetc(fr) != '\n');
	    }
	    c = fgetc(fr);
	}

	fscanf(fr, " =\n");
	c = fgetc(fr);

	while (c == (int)'{')
	{
	    int u_id, v_id;
	    fscanf(fr, "%d,%d", &u_id, &v_id);

	    #ifdef sVERBOSE
	    {
	      printf("%d,%d\n", u_id, v_id);
	    }
	    #endif

	    add_Edge(u_id, v_id);
	    if (c != '\n')
	    {
		while(fgetc(fr) != '\n');
	    }
	    c = fgetc(fr);
	}

	return sRESULT_SUCCESS;
    }


    sResult sUndirectedGraph::to_File_multirobot(const sString &filename, const sString &indent) const
    {
	FILE *fw;

	if ((fw = fopen(filename.c_str(), "w")) == NULL)
	{
	    return sUNDIRECTED_GRAPH_OPEN_ERROR;
	}
	
	to_Stream_multirobot(fw, indent);
	fclose(fw);

	return sRESULT_SUCCESS;
    }


    void sUndirectedGraph::to_Stream_multirobot(FILE *fw, const sString &indent) const
    {
	fprintf(fw, "%sE =\n", indent.c_str());

	for (Edges_list::const_iterator edge = m_Edges.begin(); edge != m_Edges.end(); ++edge)
	{
	    fprintf(fw, "%s{%d,%d} (-1)\n", indent.c_str(), edge->m_arc_uv.m_target->m_id, edge->m_arc_vu.m_target->m_id);
	}
    }


/*----------------------------------------------------------------------------*/

} // namespace sReloc
